import fs from 'fs';
import path from 'path';
import ts from 'typescript';
import { ExtensionsUtil, separator } from './const.js';
import { asString } from './type.js';
const formProperFilePath = (props) => {
    return props.filePath.split(separator).filter(Boolean).join(separator);
};
const checkJavaScriptFileExistByAppend = (props) => {
    const { js } = ExtensionsUtil.extensions.javaScript;
    const jsFilePath = `${props.filePath}${js}`;
    if (fs.existsSync(jsFilePath)) {
        return { extension: js, filePath: jsFilePath };
    }
    return false;
};
const checkTypeDefinitionFileExistByAppend = (props) => {
    const { js } = ExtensionsUtil.extensions.javaScript;
    const { dts } = ExtensionsUtil.extensions.typeDefinition;
    const dtsFilePath = `${props.filePath}${dts}`;
    if (fs.existsSync(dtsFilePath)) {
        return { extension: js, filePath: dtsFilePath };
    }
    return false;
};
const isDirectory = (filePath) => {
    try {
        return fs.lstatSync(filePath).isDirectory();
    }
    catch (error) {
        return false;
    }
};
const addJSExtensionConditionally = (props) => {
    const check = props.checkType === 'js'
        ? checkJavaScriptFileExistByAppend
        : checkTypeDefinitionFileExistByAppend;
    const skip = {
        procedure: 'skip',
    };
    if (!isDirectory(props.filePath)) {
        const extensionResult = check({
            filePath: props.filePath,
        });
        if (!extensionResult) {
            return skip;
        }
        return {
            procedure: 'proceed',
            absolutePath: extensionResult.filePath,
            importPath: formProperFilePath({
                filePath: `${props.importPath}${extensionResult.extension}`,
            }),
        };
    }
    const extensionResult = check({
        filePath: path.posix.join(props.filePath, 'index'),
    });
    if (!extensionResult) {
        return skip;
    }
    const file = `index${extensionResult.extension}`;
    return {
        procedure: 'proceed',
        absolutePath: extensionResult.filePath,
        importPath: formProperFilePath({
            filePath: [props.importPath, separator, file].join(''),
        }),
    };
};
const addJSExtension = (props) => {
    if (ExtensionsUtil.matchJs(props.filePath)) {
        return {
            procedure: 'skip',
        };
    }
    const result = addJSExtensionConditionally({
        ...props,
        checkType: 'js',
    });
    if (result.procedure === 'proceed') {
        return result;
    }
    return addJSExtensionConditionally({
        ...props,
        checkType: 'dts',
    });
};
const traverseAndUpdateFileWithJSExtension = (files) => {
    return ({ parsed, code }) => {
        const delimiter = '';
        const characters = code.split(delimiter);
        const file = parsed.fileName;
        const replaceNodes = parsed.statements.flatMap((statement) => {
            switch (statement.kind) {
                case ts.SyntaxKind.ImportDeclaration:
                case ts.SyntaxKind.ExportDeclaration: {
                    const importExportDeclaration = statement;
                    if (!importExportDeclaration.moduleSpecifier) {
                        return [];
                    }
                    const moduleSpecifier = asString({
                        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                        // @ts-ignore
                        value: importExportDeclaration.moduleSpecifier.text,
                        error: new Error('Module specifier of node should have text'),
                    });
                    const fileName = formProperFilePath({
                        filePath: !moduleSpecifier.endsWith(separator)
                            ? moduleSpecifier
                            : moduleSpecifier.slice(0, -1),
                    })
                        .split(separator)
                        .at(-1);
                    if (!fileName) {
                        throw new Error(`Impossible for file name to be non-existent for ${moduleSpecifier}`);
                    }
                    if (moduleSpecifier.startsWith('.')) {
                        const result = addJSExtension({
                            importPath: moduleSpecifier,
                            filePath: path.posix.join(file, '..', moduleSpecifier),
                        });
                        switch (result.procedure) {
                            case 'proceed': {
                                // if file name not included in list of js file read
                                if (files.find((file) => {
                                    return file.endsWith(result.absolutePath);
                                })) {
                                    const before = characters
                                        .filter((_, index) => {
                                        return (index >
                                            importExportDeclaration.pos &&
                                            index <
                                                importExportDeclaration.end);
                                    })
                                        .join(delimiter);
                                    return [
                                        {
                                            before,
                                            after: before.replace(moduleSpecifier, result.importPath),
                                        },
                                    ];
                                }
                            }
                        }
                    }
                }
            }
            return [];
        });
        return !replaceNodes.length
            ? []
            : [
                {
                    file,
                    code: replaceNodes.reduce((code, node) => {
                        return code.replace(node.before, node.after);
                    }, code),
                },
            ];
    };
};
export default traverseAndUpdateFileWithJSExtension;
//# sourceMappingURL=traverse-and-update.js.map